from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="tastytrade-api",
    version="1.1.0",
    author="Peter Oroszvari",
    author_email="peter@oroszvari.hu",
    description="A Python client and SDK for the Tastytrade API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/peter-oroszvari/tastytrade-api",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",

    ],
    python_requires=">=3.6",
    install_requires=[
        "requests",
        "websocket-client",
        "websockets"
    ],
)
